@@ -21,6 +21,8 @@
 #include <fcntl.h>
 #include <getopt.h>
 #include <linux/f2fs.h>
+#include <linux/fs.h>
+#include <malloc.h>
 #include <math.h>
 #include <sched.h>
 #include <stdarg.h>
@@ -87,6 +89,7 @@ class Logcat {
     int Run(int argc, char** argv);
 
   private:
+    FILE* OpenLogFile(const char* path);
     void RotateLogs();
     void ProcessBuffer(struct log_msg* buf);
     LogcatPriorityProto GetProtoPriority(const AndroidLogEntry& entry);
@@ -100,6 +103,8 @@ class Logcat {
         }
     }
 
+    const bool kCompressLogcat = android::base::GetBoolProperty("ro.logcat.compress", false);
+
     // Used for all options
     std::unique_ptr<AndroidLogFormat, decltype(&android_log_format_free)> logformat_{
             android_log_format_new(), &android_log_format_free};
@@ -136,29 +141,30 @@ class Logcat {
     ProcessNames process_names_;
 };
 
-static void pinLogFile(int fd, size_t sizeKB) {
+static void startCompMode(int fd) {
     // Ignore errors.
-    uint32_t set = 1;
-    ioctl(fd, F2FS_IOC_SET_PIN_FILE, &set);
-    fallocate(fd, FALLOC_FL_KEEP_SIZE, 0, (sizeKB << 10));
+    long flag = FS_COMPR_FL;
+    ioctl(fd, FS_IOC_SETFLAGS, &flag);
 }
 
-static void unpinLogFile(const char* pathname) {
-    int fd = open(pathname, O_WRONLY | O_CLOEXEC);
+static void releaseCompBlocks(const char* pathname) {
+    int fd = open(pathname, O_RDONLY);
     if (fd != -1) {
         // Ignore errors.
-        uint32_t set = 0;
-        ioctl(fd, F2FS_IOC_SET_PIN_FILE, &set);
+        unsigned long long blkcnt;
+        ioctl(fd, F2FS_IOC_RELEASE_COMPRESS_BLOCKS, &blkcnt);
         close(fd);
     }
 }
 
-static FILE* openLogFile(const char* path, size_t sizeKB) {
+FILE* Logcat::OpenLogFile(const char* path) {
     int fd = open(path, O_WRONLY | O_APPEND | O_CREAT | O_CLOEXEC, S_IRUSR | S_IWUSR | S_IRGRP);
     if (fd == -1) {
         error(EXIT_FAILURE, errno, "couldn't open output file '%s'", path);
     }
-    pinLogFile(fd, sizeKB);
+    if (kCompressLogcat) {
+        startCompMode(fd);
+    }
     return fdopen(fd, "w");
 }
 
@@ -169,6 +175,10 @@ void Logcat::RotateLogs() {
     fclose(output_file_);
     output_file_ = nullptr;
 
+    if (kCompressLogcat) {
+        releaseCompBlocks(output_file_name_);
+    }
+
     // Compute the maximum number of digits needed to count up to
     // maxRotatedLogs in decimal.  eg:
     // maxRotatedLogs == 30
@@ -193,15 +203,13 @@ void Logcat::RotateLogs() {
             break;
         }
 
-        unpinLogFile(file0.c_str());
-
         if (rename(file0.c_str(), file1.c_str()) == -1 && errno != ENOENT) {
             error(0, errno, "rename('%s', '%s') failed while rotating log files", file0.c_str(),
                   file1.c_str());
         }
     }
 
-    output_file_ = openLogFile(output_file_name_, log_rotate_size_kb_);
+    output_file_ = OpenLogFile(output_file_name_);
     out_byte_count_ = 0;
 }
 
@@ -346,7 +354,7 @@ void Logcat::SetupOutputAndSchedulingPolicy(bool blocking) {
         }
     }
 
-    output_file_ = openLogFile(output_file_name_, log_rotate_size_kb_);
+    output_file_ = OpenLogFile(output_file_name_);
 
     struct stat sb;
     if (fstat(fileno(output_file_), &sb) == -1) {
@@ -1148,6 +1156,11 @@ int Logcat::Run(int argc, char** argv) {
     bool blocking = !(mode & ANDROID_LOG_NONBLOCK);
     SetupOutputAndSchedulingPolicy(blocking);
 
+    // Purge as much memory as possible before going into the log reading loop.
+    // Do this before checking if logd is ready just in case logd isn't
+    // ready and this call can be done without penalizing start up.
+    mallopt(M_PURGE_ALL, 0);
+
     if (!WaitForProperty("logd.ready", "true", std::chrono::seconds(1))) {
         error(EXIT_FAILURE, 0, "Failed to wait for logd.ready to become true. logd not running?");
     }
@@ -1165,8 +1178,13 @@ If you have enabled significant logging, look into using the -G option to increa
         }
 
         if (ret < 0) {
-            if (ret == -EAGAIN) break;
-
+            if (ret == -EAGAIN || ret == -EWOULDBLOCK || ret == -ETIMEDOUT) {
+                // For non-blocking mode, a socket with a 2s timeout is used to read logs.
+                // Either recv returned -EAGAIN or -EWOULDBLOCK (see man recv)
+                // or connect returned -EAGAIN or -ETIMEOUT.
+                // In either case, the caller should call logcat again at a later time.
+                break;
+            }
             if (ret == -EIO) {
                 error(EXIT_FAILURE, 0, "Unexpected EOF!");
             }
