@@ -14,8 +14,6 @@
  * limitations under the License.
  */
 
-#if !ADB_HOST
-
 #if !defined(__ANDROID_RECOVERY__)
 #define TRACE_TAG JDWP
 
@@ -461,12 +459,16 @@ static int jdwp_tracker_enqueue(asocket* s, apacket::payload_type) {
 }
 
 static asocket* create_process_tracker_service_socket(TrackerKind kind) {
-    auto t = std::make_unique<JdwpTracker>(kind, true);
+    std::unique_ptr<JdwpTracker> t = std::make_unique<JdwpTracker>(kind, true);
     if (!t) {
         LOG(FATAL) << "failed to allocate JdwpTracker";
     }
 
-    memset(t.get(), 0, sizeof(asocket));
+    /* Object layout (with an inheritance hierarchy) varies across arch (e.g
+     * armv7a/Android TV vs aarch64), so no assumptions can be made about
+     * accessing fields based on offsets (e.g memset(t.get(), 0, sizeof(asocket))
+     * might clobber an unintended memory location).
+     */
 
     install_local_socket(t.get());
     D("LS(%d): created new jdwp tracker service", t->id);
@@ -495,7 +497,7 @@ int init_jdwp(void) {
         adb_thread_setname("jdwp control");
         adbconnection_listen([](int fd, ProcessInfo process) {
             LOG(INFO) << "jdwp connection from " << process.pid;
-            fdevent_run_on_main_thread([fd, process] {
+            fdevent_run_on_looper([fd, process] {
                 unique_fd ufd(fd);
                 auto proc = std::make_unique<JdwpProcess>(std::move(ufd), process);
                 if (!proc) {
@@ -534,4 +536,3 @@ int init_jdwp() {
 }
 
 #endif /* defined(__ANDROID_RECOVERY__) */
-#endif /* !ADB_HOST */
