@@ -18,6 +18,8 @@
 
 #include <sys/types.h>
 
+#include <algorithm>
+
 #include "adb_unique_fd.h"
 
 #if defined(_WIN32)
@@ -35,7 +37,7 @@ ssize_t adb_writev(borrowed_fd fd, const adb_iovec* iov, int iovcnt);
 #include <sys/uio.h>
 using adb_iovec = struct iovec;
 inline ssize_t adb_writev(borrowed_fd fd, const adb_iovec* iov, int iovcnt) {
-    return writev(fd.get(), iov, iovcnt);
+    return writev(fd.get(), iov, std::min(iovcnt, IOV_MAX));
 }
 
 #endif
