@@ -1056,7 +1056,7 @@ sp<IBinder> SurfaceComposerClient::createDisplay(const String8& displayName, boo
     sp<IBinder> display = nullptr;
     binder::Status status =
             ComposerServiceAIDL::getComposerService()->createDisplay(std::string(
-                                                                             displayName.string()),
+                                                                             displayName.c_str()),
                                                                      secure, &display);
     return status.isOk() ? display : nullptr;
 }
