@@ -27,16 +27,21 @@
 //! available to tweak it via a config file.
 
 mod bp;
-mod cargo_out;
+mod cargo;
+mod config;
 
+use crate::config::legacy;
+use crate::config::Config;
+use crate::config::PackageConfig;
 use anyhow::bail;
 use anyhow::Context;
 use anyhow::Result;
 use bp::*;
-use cargo_out::*;
+use cargo::{
+    cargo_out::parse_cargo_out, metadata::parse_cargo_metadata_file, Crate, CrateType, ExternType,
+};
 use clap::Parser;
-use once_cell::sync::Lazy;
-use regex::Regex;
+use clap::Subcommand;
 use std::collections::BTreeMap;
 use std::collections::VecDeque;
 use std::fs::File;
@@ -49,8 +54,8 @@ use std::process::Command;
 //  * handle errors, esp. in cargo.out parsing. they should fail the program with an error code
 //  * handle warnings. put them in comments in the android.bp, some kind of report section
 
+/// Command-line parameters for `cargo_embargo`.
 #[derive(Parser, Debug)]
-#[clap()]
 struct Args {
     /// Use the cargo binary in the `cargo_bin` directory. Defaults to cargo in $PATH.
     ///
@@ -64,83 +69,19 @@ struct Args {
     /// available.
     #[clap(long)]
     reuse_cargo_out: bool,
+    #[command(subcommand)]
+    mode: Option<Mode>,
 }
 
-fn default_apex_available() -> Vec<String> {
-    vec!["//apex_available:platform".to_string(), "//apex_available:anyapex".to_string()]
-}
-
-/// Options that apply to everything.
-#[derive(serde::Deserialize)]
-#[serde(deny_unknown_fields)]
-struct Config {
-    /// Whether to output "rust_test" modules.
-    tests: bool,
-    /// Set of features to enable. If non-empty, disables the default crate features.
-    #[serde(default)]
-    features: Vec<String>,
-    /// Whether to build with --workspace.
-    #[serde(default)]
-    workspace: bool,
-    /// When workspace is enabled, list of --exclude crates.
-    #[serde(default)]
-    workspace_excludes: Vec<String>,
-    /// Value to use for every generated module's "defaults" field.
-    global_defaults: Option<String>,
-    /// Value to use for every generated library module's "apex_available" field.
-    #[serde(default = "default_apex_available")]
-    apex_available: Vec<String>,
-    /// Map of renames for modules. For example, if a "libfoo" would be generated and there is an
-    /// entry ("libfoo", "libbar"), the generated module will be called "libbar" instead.
-    ///
-    /// Also, affects references to dependencies (e.g. in a "static_libs" list), even those outside
-    /// the project being processed.
-    #[serde(default)]
-    module_name_overrides: BTreeMap<String, String>,
-    /// Package specific config options.
-    #[serde(default)]
-    package: BTreeMap<String, PackageConfig>,
-    /// Modules in this list will not be generated.
-    #[serde(default)]
-    module_blocklist: Vec<String>,
-    /// Modules name => Soong "visibility" property.
-    #[serde(default)]
-    module_visibility: BTreeMap<String, Vec<String>>,
-}
-
-/// Options that apply to everything in a package (i.e. everything associated with a particular
-/// Cargo.toml file).
-#[derive(serde::Deserialize, Default)]
-#[serde(deny_unknown_fields)]
-struct PackageConfig {
-    /// Whether to compile for device. Defaults to true.
-    #[serde(default)]
-    device_supported: Option<bool>,
-    /// Whether to compile for host. Defaults to true.
-    #[serde(default)]
-    host_supported: Option<bool>,
-    /// Generate "rust_library_rlib" instead of "rust_library".
-    #[serde(default)]
-    force_rlib: bool,
-    /// Whether to disable "unit_test" for "rust_test" modules.
-    // TODO: Should probably be a list of modules or crates. A package might have a mix of unit and
-    // integration tests.
-    #[serde(default)]
-    no_presubmit: bool,
-    /// File with content to append to the end of the generated Android.bp.
-    add_toplevel_block: Option<PathBuf>,
-    /// File with content to append to the end of each generated module.
-    add_module_block: Option<PathBuf>,
-    /// Modules in this list will not be added as dependencies of generated modules.
-    #[serde(default)]
-    dep_blocklist: Vec<String>,
-    /// Patch file to apply after Android.bp is generated.
-    patch: Option<PathBuf>,
-    /// Copy build.rs output to ./out/* and add a genrule to copy ./out/* to genrule output.
-    /// For crates with code pattern:
-    ///     include!(concat!(env!("OUT_DIR"), "/<some_file>.rs"))
-    #[serde(default)]
-    copy_out: bool,
+#[derive(Clone, Debug, Subcommand)]
+enum Mode {
+    /// Converts a legacy `cargo2android.json` config file to the equivalent `cargo_embargo.json`
+    /// config.
+    Convert {
+        package_name: String,
+        #[arg(long)]
+        no_build: bool,
+    },
 }
 
 fn main() -> Result<()> {
@@ -150,7 +91,27 @@ fn main() -> Result<()> {
         .with_context(|| format!("failed to read file: {:?}", args.cfg))?;
     // Add some basic support for comments to JSON.
     let json_str: String = json_str.lines().filter(|l| !l.trim_start().starts_with("//")).collect();
-    let cfg: Config = serde_json::from_str(&json_str).context("failed to parse config")?;
+
+    match args.mode {
+        Some(Mode::Convert { package_name, no_build }) => {
+            let legacy_config: legacy::Config =
+                serde_json::from_str(&json_str).context("failed to parse legacy config")?;
+            let new_config = legacy_config.to_embargo(&package_name, !no_build)?;
+            let new_config_str = serde_json::to_string_pretty(&new_config)
+                .context("failed to serialize new config")?;
+            println!("{}", new_config_str);
+        }
+        None => {
+            run_embargo(args, &json_str)?;
+        }
+    }
+
+    Ok(())
+}
+
+/// Runs cargo_embargo with the given JSON configuration string.
+fn run_embargo(args: Args, json_str: &str) -> Result<()> {
+    let cfg: Config = serde_json::from_str(json_str).context("failed to parse config")?;
 
     if !Path::new("Cargo.toml").try_exists().context("when checking Cargo.toml")? {
         bail!("Cargo.toml missing. Run in a directory with a Cargo.toml file.");
@@ -175,8 +136,11 @@ fn main() -> Result<()> {
             .context("generate_cargo_out failed")?;
     }
 
-    let crates =
-        parse_cargo_out(cargo_out_path, cargo_metadata_path).context("parse_cargo_out failed")?;
+    let crates = if cfg.run_cargo {
+        parse_cargo_out(cargo_out_path, cargo_metadata_path).context("parse_cargo_out failed")?
+    } else {
+        parse_cargo_metadata_file(cargo_metadata_path, &cfg)?
+    };
 
     // Find out files.
     // Example: target.tmp/x86_64-unknown-linux-gnu/debug/build/metrics-d2dd799cebf1888d/out/event_details.rs
@@ -200,15 +164,29 @@ fn main() -> Result<()> {
         }
     }
 
-    // Group by package.
+    write_all_bp(&cfg, crates, &package_out_files)
+}
+
+fn group_by_package(crates: Vec<Crate>) -> BTreeMap<PathBuf, Vec<Crate>> {
     let mut module_by_package: BTreeMap<PathBuf, Vec<Crate>> = BTreeMap::new();
     for c in crates {
         module_by_package.entry(c.package_dir.clone()).or_default().push(c);
     }
+    module_by_package
+}
+
+fn write_all_bp(
+    cfg: &Config,
+    crates: Vec<Crate>,
+    package_out_files: &BTreeMap<String, Vec<PathBuf>>,
+) -> Result<()> {
+    // Group by package.
+    let module_by_package = group_by_package(crates);
+
     // Write an Android.bp file per package.
     for (package_dir, crates) in module_by_package {
         write_android_bp(
-            &cfg,
+            cfg,
             package_dir,
             &crates,
             package_out_files.get(&crates[0].package_name),
@@ -272,28 +250,30 @@ fn generate_cargo_out(cfg: &Config, cargo_out_path: &str, cargo_metadata_path: &
             .args(&feature_args),
     )?;
 
-    // cargo build
-    run_cargo(
-        &mut cargo_out_file,
-        Command::new("cargo")
-            .args(["build", "--target", default_target])
-            .args(verbose_args)
-            .args(target_dir_args)
-            .args(&workspace_args)
-            .args(&feature_args),
-    )?;
-
-    if cfg.tests {
-        // cargo build --tests
+    if cfg.run_cargo {
+        // cargo build
         run_cargo(
             &mut cargo_out_file,
             Command::new("cargo")
-                .args(["build", "--target", default_target, "--tests"])
+                .args(["build", "--target", default_target])
                 .args(verbose_args)
                 .args(target_dir_args)
                 .args(&workspace_args)
                 .args(&feature_args),
         )?;
+
+        if cfg.tests {
+            // cargo build --tests
+            run_cargo(
+                &mut cargo_out_file,
+                Command::new("cargo")
+                    .args(["build", "--target", default_target, "--tests"])
+                    .args(verbose_args)
+                    .args(target_dir_args)
+                    .args(&workspace_args)
+                    .args(&feature_args),
+            )?;
+        }
     }
 
     Ok(())
@@ -324,6 +304,38 @@ fn write_android_bp(
         Err(e) => bail!("error when reading {bp_path:?}: {e}"),
     };
 
+    // If `copy_out` is enabled and there are any generated out files for the package, copy them to
+    // the appropriate directory.
+    if let (true, Some(out_files)) = (package_cfg.copy_out, out_files) {
+        let out_dir = package_dir.join("out");
+        if !out_dir.exists() {
+            std::fs::create_dir(&out_dir).expect("failed to create out dir");
+        }
+
+        for f in out_files.iter() {
+            let dest = out_dir.join(f.file_name().unwrap());
+            std::fs::copy(f, dest).expect("failed to copy out file");
+        }
+    }
+
+    if let Some(bp_contents) =
+        generate_android_bp(&license_section, cfg, package_cfg, &package_name, crates, out_files)?
+    {
+        write_format_android_bp(&bp_path, &bp_contents, package_cfg.patch.as_deref())?;
+    }
+
+    Ok(())
+}
+
+/// Generates and returns a Soong Blueprint for the given set of crates.
+fn generate_android_bp(
+    license_section: &str,
+    cfg: &Config,
+    package_cfg: &PackageConfig,
+    package_name: &str,
+    crates: &[Crate],
+    out_files: Option<&Vec<PathBuf>>,
+) -> Result<Option<String>> {
     let mut bp_contents = String::new();
     bp_contents += "// This file is generated by cargo_embargo.\n";
     bp_contents += "// Do not modify this file as changes will be overridden on upgrade.\n\n";
@@ -332,31 +344,23 @@ fn write_android_bp(
 
     let mut modules = Vec::new();
 
-    let extra_srcs = match (package_cfg.copy_out, out_files) {
-        (true, Some(out_files)) => {
-            let out_dir = package_dir.join("out");
-            if !out_dir.exists() {
-                std::fs::create_dir(&out_dir).expect("failed to create out dir");
-            }
+    let extra_srcs = if let (true, Some(out_files)) = (package_cfg.copy_out, out_files) {
+        let outs: Vec<String> = out_files
+            .iter()
+            .map(|f| f.file_name().unwrap().to_str().unwrap().to_string())
+            .collect();
 
-            let mut outs: Vec<String> = Vec::new();
-            for f in out_files.iter() {
-                let dest = out_dir.join(f.file_name().unwrap());
-                std::fs::copy(f, dest).expect("failed to copy out file");
-                outs.push(f.file_name().unwrap().to_str().unwrap().to_string());
-            }
+        let mut m = BpModule::new("genrule".to_string());
+        let module_name = format!("copy_{}_build_out", package_name);
+        m.props.set("name", module_name.clone());
+        m.props.set("srcs", vec!["out/*"]);
+        m.props.set("cmd", "cp $(in) $(genDir)");
+        m.props.set("out", outs);
+        modules.push(m);
 
-            let mut m = BpModule::new("genrule".to_string());
-            let module_name = format!("copy_{}_build_out", package_name);
-            m.props.set("name", module_name.clone());
-            m.props.set("srcs", vec!["out/*"]);
-            m.props.set("cmd", "cp $(in) $(genDir)");
-            m.props.set("out", outs);
-            modules.push(m);
-
-            vec![":".to_string() + &module_name]
-        }
-        _ => vec![],
+        vec![":".to_string() + &module_name]
+    } else {
+        vec![]
     };
 
     for c in crates {
@@ -370,7 +374,7 @@ fn write_android_bp(
         )?);
     }
     if modules.is_empty() {
-        return Ok(());
+        return Ok(None);
     }
 
     // In some cases there are nearly identical rustc invocations that that get processed into
@@ -389,9 +393,19 @@ fn write_android_bp(
             &std::fs::read_to_string(path).with_context(|| format!("failed to read {path:?}"))?;
         bp_contents += "\n";
     }
-    File::create(&bp_path)?.write_all(bp_contents.as_bytes())?;
+    Ok(Some(bp_contents))
+}
 
-    let bpfmt_output = Command::new("bpfmt").arg("-w").arg(&bp_path).output()?;
+/// Writes the given contents to the given `Android.bp` file, formats it with `bpfmt`, and applies
+/// the patch if there is one.
+fn write_format_android_bp(
+    bp_path: &Path,
+    bp_contents: &str,
+    patch_path: Option<&Path>,
+) -> Result<()> {
+    File::create(bp_path)?.write_all(bp_contents.as_bytes())?;
+
+    let bpfmt_output = Command::new("bpfmt").arg("-w").arg(bp_path).output()?;
     if !bpfmt_output.status.success() {
         eprintln!(
             "WARNING: bpfmt -w {:?} failed before patch: {}",
@@ -400,14 +414,13 @@ fn write_android_bp(
         );
     }
 
-    if let Some(patch_path) = &package_cfg.patch {
-        let patch_output =
-            Command::new("patch").arg("-s").arg(&bp_path).arg(patch_path).output()?;
+    if let Some(patch_path) = patch_path {
+        let patch_output = Command::new("patch").arg("-s").arg(bp_path).arg(patch_path).output()?;
         if !patch_output.status.success() {
             eprintln!("WARNING: failed to apply patch {:?}", patch_path);
         }
         // Re-run bpfmt after the patch so
-        let bpfmt_output = Command::new("bpfmt").arg("-w").arg(&bp_path).output()?;
+        let bpfmt_output = Command::new("bpfmt").arg("-w").arg(bp_path).output()?;
         if !bpfmt_output.status.success() {
             eprintln!(
                 "WARNING: bpfmt -w {:?} failed after patch: {}",
@@ -431,7 +444,7 @@ fn crate_to_bp_modules(
 ) -> Result<Vec<BpModule>> {
     let mut modules = Vec::new();
     for crate_type in &crate_.types {
-        let host = if package_cfg.device_supported.unwrap_or(true) { "" } else { "_host" };
+        let host = if package_cfg.device_supported { "" } else { "_host" };
         let rlib = if package_cfg.force_rlib { "_rlib" } else { "" };
         let (module_type, module_name, stem) = match crate_type {
             CrateType::Bin => {
@@ -486,13 +499,17 @@ fn crate_to_bp_modules(
             m.props.set("defaults", vec![defaults.clone()]);
         }
 
-        if package_cfg.host_supported.unwrap_or(true)
-            && package_cfg.device_supported.unwrap_or(true)
+        if package_cfg.host_supported
+            && package_cfg.device_supported
             && module_type != "rust_proc_macro"
         {
             m.props.set("host_supported", true);
         }
 
+        if !crate_type.is_test() && package_cfg.host_supported && package_cfg.host_first_multilib {
+            m.props.set("compile_multilib", "first");
+        }
+
         m.props.set("crate_name", crate_.name.clone());
         m.props.set("cargo_env_compat", true);
 
@@ -503,7 +520,7 @@ fn crate_to_bp_modules(
         if crate_.types.contains(&CrateType::Test) {
             m.props.set("test_suites", vec!["general-tests"]);
             m.props.set("auto_gen_config", true);
-            if package_cfg.host_supported.unwrap_or(true) {
+            if package_cfg.host_supported {
                 m.props.object("test_options").set("unit_test", !package_cfg.no_presubmit);
             }
         }
@@ -531,27 +548,10 @@ fn crate_to_bp_modules(
 
         let mut rust_libs = Vec::new();
         let mut proc_macro_libs = Vec::new();
-        for (extern_name, filename) in &crate_.externs {
-            if extern_name == "proc_macro" {
-                continue;
-            }
-            let filename =
-                filename.as_ref().unwrap_or_else(|| panic!("no filename for {}", extern_name));
-            // Example filename: "libgetrandom-fd8800939535fc59.rmeta"
-            static REGEX: Lazy<Regex> =
-                Lazy::new(|| Regex::new(r"^lib(.*)-[0-9a-f]*.(rlib|so|rmeta)$").unwrap());
-            let lib_name = if let Some(x) = REGEX.captures(filename).and_then(|x| x.get(1)) {
-                x
-            } else {
-                bail!("bad filename for extern {}: {}", extern_name, filename);
-            };
-            if filename.ends_with(".rlib") || filename.ends_with(".rmeta") {
-                rust_libs.push(lib_name.as_str().to_string());
-            } else if filename.ends_with(".so") {
-                // Assume .so files are always proc_macros. May not always be right.
-                proc_macro_libs.push(lib_name.as_str().to_string());
-            } else {
-                unreachable!();
+        for extern_dep in &crate_.externs {
+            match extern_dep.extern_type {
+                ExternType::Rust => rust_libs.push(extern_dep.lib_name.clone()),
+                ExternType::ProcMacro => proc_macro_libs.push(extern_dep.lib_name.clone()),
             }
         }
 
@@ -583,17 +583,29 @@ fn crate_to_bp_modules(
             m.props.set("shared_libs", process_lib_deps(crate_.shared_libs.clone()));
         }
 
-        if !cfg.apex_available.is_empty()
-            && [
-                CrateType::Lib,
-                CrateType::RLib,
-                CrateType::DyLib,
-                CrateType::CDyLib,
-                CrateType::StaticLib,
-            ]
-            .contains(crate_type)
-        {
-            m.props.set("apex_available", cfg.apex_available.clone());
+        if crate_type.is_library() {
+            if !cfg.apex_available.is_empty() {
+                m.props.set("apex_available", cfg.apex_available.clone());
+            }
+            if cfg.product_available {
+                m.props.set("product_available", true);
+            }
+            if cfg.vendor_available {
+                m.props.set("vendor_available", true);
+            }
+
+            if package_cfg.device_supported {
+                if let Some(min_sdk_version) = &cfg.min_sdk_version {
+                    m.props.set("min_sdk_version", min_sdk_version.clone());
+                }
+            }
+        }
+        if crate_type.is_test() {
+            if let Some(data) =
+                package_cfg.test_data.get(crate_.main_src.to_string_lossy().as_ref())
+            {
+                m.props.set("data", data.clone());
+            }
         }
 
         if let Some(visibility) = cfg.module_visibility.get(module_name) {
@@ -610,3 +622,136 @@ fn crate_to_bp_modules(
     }
     Ok(modules)
 }
+
+#[cfg(test)]
+mod tests {
+    use super::*;
+    use std::env::{current_dir, set_current_dir};
+    use std::fs::{self, read_to_string};
+    use std::path::PathBuf;
+
+    const TESTDATA_PATH: &str = "testdata";
+
+    #[test]
+    fn generate_bp() {
+        for testdata_directory_path in testdata_directories() {
+            let cfg: Config = serde_json::from_reader(
+                File::open(testdata_directory_path.join("cargo_embargo.json"))
+                    .expect("Failed to open cargo_embargo.json"),
+            )
+            .unwrap();
+            let crates: Vec<Crate> = serde_json::from_reader(
+                File::open(testdata_directory_path.join("crates.json"))
+                    .expect("Failed to open crates.json"),
+            )
+            .unwrap();
+            let expected_output =
+                read_to_string(testdata_directory_path.join("expected_Android.bp")).unwrap();
+
+            let module_by_package = group_by_package(crates);
+            assert_eq!(module_by_package.len(), 1);
+            let crates = module_by_package.into_values().next().unwrap();
+            let package_name = &crates[0].package_name;
+            let def = PackageConfig::default();
+            let package_cfg = cfg.package.get(package_name).unwrap_or(&def);
+
+            let old_current_dir = current_dir().unwrap();
+            set_current_dir(&testdata_directory_path).unwrap();
+
+            let output = generate_android_bp(
+                "// License section.\n",
+                &cfg,
+                package_cfg,
+                package_name,
+                &crates,
+                None,
+            )
+            .unwrap()
+            .unwrap();
+
+            assert_eq!(output, expected_output);
+
+            set_current_dir(old_current_dir).unwrap();
+        }
+    }
+
+    #[test]
+    fn crate_to_bp_empty() {
+        let c = Crate {
+            name: "name".to_string(),
+            package_name: "package_name".to_string(),
+            edition: "2021".to_string(),
+            types: vec![],
+            ..Default::default()
+        };
+        let cfg = Config { ..Default::default() };
+        let package_cfg = PackageConfig { ..Default::default() };
+        let modules = crate_to_bp_modules(&c, &cfg, &package_cfg, &[]).unwrap();
+
+        assert_eq!(modules, vec![]);
+    }
+
+    #[test]
+    fn crate_to_bp_minimal() {
+        let c = Crate {
+            name: "name".to_string(),
+            package_name: "package_name".to_string(),
+            edition: "2021".to_string(),
+            types: vec![CrateType::Lib],
+            ..Default::default()
+        };
+        let cfg = Config { ..Default::default() };
+        let package_cfg = PackageConfig { ..Default::default() };
+        let modules = crate_to_bp_modules(&c, &cfg, &package_cfg, &[]).unwrap();
+
+        assert_eq!(
+            modules,
+            vec![BpModule {
+                module_type: "rust_library".to_string(),
+                props: BpProperties {
+                    map: [
+                        (
+                            "apex_available".to_string(),
+                            BpValue::List(vec![
+                                BpValue::String("//apex_available:platform".to_string()),
+                                BpValue::String("//apex_available:anyapex".to_string()),
+                            ])
+                        ),
+                        ("cargo_env_compat".to_string(), BpValue::Bool(true)),
+                        ("crate_name".to_string(), BpValue::String("name".to_string())),
+                        ("edition".to_string(), BpValue::String("2021".to_string())),
+                        ("host_supported".to_string(), BpValue::Bool(true)),
+                        ("name".to_string(), BpValue::String("libname".to_string())),
+                        ("product_available".to_string(), BpValue::Bool(true)),
+                        ("srcs".to_string(), BpValue::List(vec![BpValue::String("".to_string())])),
+                        ("vendor_available".to_string(), BpValue::Bool(true)),
+                    ]
+                    .into_iter()
+                    .collect(),
+                    raw_block: None
+                }
+            }]
+        );
+    }
+
+    /// Returns a list of directories containing test data.
+    ///
+    /// Each directory under `testdata/` contains a single test case.
+    pub fn testdata_directories() -> Vec<PathBuf> {
+        fs::read_dir(TESTDATA_PATH)
+            .expect("Failed to read testdata directory")
+            .filter_map(|entry| {
+                let entry = entry.expect("Error reading testdata directory entry");
+                if entry
+                    .file_type()
+                    .expect("Error getting metadata for testdata subdirectory")
+                    .is_dir()
+                {
+                    Some(entry.path())
+                } else {
+                    None
+                }
+            })
+            .collect()
+    }
+}
