@@ -18,7 +18,7 @@ enum TransitionType {
   ROTATION = 'ROTATION',
   PIP_ENTER = 'PIP_ENTER',
   PIP_RESIZE ='PIP_RESIZE',
-  PIP_CLOSE = 'PIP_CLOSE',
+  PIP_EXPAND = 'PIP_EXPAND',
   PIP_EXIT = 'PIP_EXIT',
   APP_LAUNCH = 'APP_LAUNCH',
   APP_CLOSE = 'APP_CLOSE',
