@@ -17,6 +17,7 @@
 
 #include <llvm/ADT/Optional.h>
 
+#include <set>
 #include <string>
 #include <vector>
 
@@ -25,6 +26,9 @@ namespace header_checker {
 namespace utils {
 
 
+// This comparison function allows finding elements by string_view.
+using StringSet = std::set<std::string, std::less<>>;
+
 std::string_view Trim(std::string_view s);
 
 bool StartsWith(std::string_view s, std::string_view prefix);
@@ -40,6 +44,8 @@ bool ParseBool(const std::string &s);
 
 bool IsGlobPattern(std::string_view s);
 
+bool HasMatchingGlobPattern(const StringSet &patterns, const char *text);
+
 
 }  // namespace utils
 }  // namespace header_checker
